@@ -2,7 +2,7 @@
 """
 Backports and helper functionality to support using new functionality.
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function
 
 import atexit
 import contextlib
@@ -19,6 +19,8 @@ from packaging import specifiers
 from .environment import MYPY_RUNNING
 from .utils import (
     call_function_with_correct_args,
+    filter_allowed_args,
+    get_allowed_args,
     get_method_args,
     nullcontext,
     suppress_setattr,
@@ -65,6 +67,7 @@ if MYPY_RUNNING:
     TCommandInstance = TypeVar("TCommandInstance")
     TCmdDict = Dict[str, Union[Tuple[str, str, str], TCommandInstance]]
     TInstallRequirement = TypeVar("TInstallRequirement")
+    TFormatControl = TypeVar("TFormatControl")
     TShimmedCmdDict = Union[TShim, TCmdDict]
     TWheelCache = TypeVar("TWheelCache")
     TPreparer = TypeVar("TPreparer")
@@ -350,6 +353,27 @@ def ensure_resolution_dirs(**kwargs):
             yield kwargs
 
 
+@contextlib.contextmanager
+def wheel_cache(
+    cache_dir=None,  # type: str
+    format_control=None,  # type: Any
+    wheel_cache_provider=None,  # type: TShimmedFunc
+    format_control_provider=None,  # type: Optional[TShimmedFunc]
+    tempdir_manager_provider=None,  # type: TShimmedFunc
+):
+    tempdir_manager_provider = resolve_possible_shim(tempdir_manager_provider)
+    wheel_cache_provider = resolve_possible_shim(wheel_cache_provider)
+    format_control_provider = resolve_possible_shim(format_control_provider)
+    if not format_control and not format_control_provider:
+        raise TypeError("Format control or provider needed for wheel cache!")
+    if not format_control:
+        format_control = format_control_provider(None, None)
+    with ExitStack() as ctx:
+        ctx.enter_context(tempdir_manager_provider())
+        wheel_cache = wheel_cache_provider(cache_dir, format_control)
+        yield wheel_cache
+
+
 def partial_command(shimmed_path, cmd_mapping=None):
     # type: (Type, Optional[TShimmedCmdDict]) -> Union[Type[TCommandInstance], functools.partial]
     """
@@ -412,7 +436,7 @@ def get_session(
         assert isinstance(install_cmd_provider, (type, functools.partial))
         install_cmd = install_cmd_provider()
     if options is None:
-        options = install_cmd.parser.parse_args([])  # type: ignore
+        options, _ = install_cmd.parser.parse_args([])  # type: ignore
     session = install_cmd._build_session(options)  # type: ignore
     assert session is not None
     atexit.register(session.close)
@@ -469,6 +493,7 @@ def get_requirement_set(
     cache_dir=None,  # type: Optional[str]
     options=None,  # type: Optional[Values]
     install_cmd_provider=None,  # type: Optional[TShimmedFunc]
+    wheel_cache_provider=None,  # type: Optional[TShimmedFunc]
 ):
     # (...) -> TRequirementSet
     """
@@ -478,6 +503,8 @@ def get_requirement_set(
     invalid parameters will be ignored if they are not needed to generate a
     requirement set on the current pip version.
 
+    :param :class:`~pip_shims.models.ShimmedPathCollection` wheel_cache_provider: A
+        context manager provider which resolves to a `WheelCache` instance
     :param install_command: A :class:`~pip._internal.commands.install.InstallCommand`
         instance which is used to generate the finder.
     :param :class:`~pip_shims.models.ShimmedPathCollection` req_set_provider: A provider
@@ -517,6 +544,7 @@ def get_requirement_set(
     :return: A new requirement set instance
     :rtype: :class:`~pip._internal.req.req_set.RequirementSet`
     """
+    wheel_cache_provider = resolve_possible_shim(wheel_cache_provider)
     req_set_provider = resolve_possible_shim(req_set_provider)
     if install_command is None:
         install_cmd_provider = resolve_possible_shim(install_cmd_provider)
@@ -544,10 +572,13 @@ def get_requirement_set(
     )
     if session is None and "session" in required_args:
         session = get_session(install_cmd=install_command, options=options)
-    results["wheel_cache"] = wheel_cache
-    results["session"] = session
-    results["wheel_download_dir"] = wheel_download_dir
-    return call_function_with_correct_args(req_set_provider, **results)
+    with ExitStack() as stack:
+        if wheel_cache is None:
+            wheel_cache = stack.enter_context(wheel_cache_provider(cache_dir=cache_dir))
+        results["wheel_cache"] = wheel_cache
+        results["session"] = session
+        results["wheel_download_dir"] = wheel_download_dir
+        return call_function_with_correct_args(req_set_provider, **results)
 
 
 def get_package_finder(
@@ -644,11 +675,11 @@ def get_package_finder(
     ):
         if target_python and not received_python:
             tags = target_python.get_tags()
-            version_impl = set([t[0] for t in tags])
+            version_impl = {t[0] for t in tags}
             # impls = set([v[:2] for v in version_impl])
             # impls.remove("py")
             # impl = next(iter(impls), "py") if not target_python
-            versions = set([v[2:] for v in version_impl])
+            versions = {v[2:] for v in version_impl}
             build_kwargs.update(
                 {
                     "platform": target_python.platform,
@@ -668,6 +699,7 @@ def get_package_finder(
 def shim_unpack(
     unpack_fn,  # type: TShimmedFunc
     download_dir,  # type str
+    tempdir_manager_provider,  # type: TShimmedFunc
     ireq=None,  # type: Optional[Any]
     link=None,  # type: Optional[Any]
     location=None,  # type Optional[str],
@@ -687,6 +719,8 @@ def shim_unpack(
     :param unpack_fn: A callable or shim referring to the pip implementation
     :type unpack_fn: Callable
     :param str download_dir: The directory to download the file to
+    :param TShimmedFunc tempdir_manager_provider: A callable or shim referring to
+        `global_tempdir_manager` function from pip or a shimmed no-op context manager
     :param Optional[:class:`~pip._internal.req.req_install.InstallRequirement`] ireq:
         an Install Requirement instance, defaults to None
     :param Optional[:class:`~pip._internal.models.link.Link`] link: A Link instance,
@@ -706,31 +740,33 @@ def shim_unpack(
     """
     unpack_fn = resolve_possible_shim(unpack_fn)
     downloader_provider = resolve_possible_shim(downloader_provider)
+    tempdir_manager_provider = resolve_possible_shim(tempdir_manager_provider)
     required_args = inspect.getargs(unpack_fn.__code__).args  # type: ignore
     unpack_kwargs = {"download_dir": download_dir}
-    if ireq:
-        if not link and ireq.link:
-            link = ireq.link
-        if only_download is None:
-            only_download = ireq.is_wheel
-        if hashes is None:
-            hashes = ireq.hashes(True)
-        if location is None and getattr(ireq, "source_dir", None):
-            location = ireq.source_dir
-    unpack_kwargs.update({"link": link, "location": location})
-    if hashes is not None and "hashes" in required_args:
-        unpack_kwargs["hashes"] = hashes
-    if "progress_bar" in required_args:
-        unpack_kwargs["progress_bar"] = progress_bar
-    if only_download is not None and "only_download" in required_args:
-        unpack_kwargs["only_download"] = only_download
-    if session is not None and "session" in required_args:
-        unpack_kwargs["session"] = session
-    if "downloader" in required_args and downloader_provider is not None:
-        assert session is not None
-        assert progress_bar is not None
-        unpack_kwargs["downloader"] = downloader_provider(session, progress_bar)
-    return unpack_fn(**unpack_kwargs)  # type: ignore
+    with tempdir_manager_provider():
+        if ireq:
+            if not link and ireq.link:
+                link = ireq.link
+            if only_download is None:
+                only_download = ireq.is_wheel
+            if hashes is None:
+                hashes = ireq.hashes(True)
+            if location is None and getattr(ireq, "source_dir", None):
+                location = ireq.source_dir
+        unpack_kwargs.update({"link": link, "location": location})
+        if hashes is not None and "hashes" in required_args:
+            unpack_kwargs["hashes"] = hashes
+        if "progress_bar" in required_args:
+            unpack_kwargs["progress_bar"] = progress_bar
+        if only_download is not None and "only_download" in required_args:
+            unpack_kwargs["only_download"] = only_download
+        if session is not None and "session" in required_args:
+            unpack_kwargs["session"] = session
+        if "downloader" in required_args and downloader_provider is not None:
+            assert session is not None
+            assert progress_bar is not None
+            unpack_kwargs["downloader"] = downloader_provider(session, progress_bar)
+        return unpack_fn(**unpack_kwargs)  # type: ignore
 
 
 def _ensure_finder(
@@ -839,7 +875,7 @@ def make_preparer(
     preparer_fn = resolve_possible_shim(preparer_fn)
     downloader_provider = resolve_possible_shim(downloader_provider)
     finder_provider = resolve_possible_shim(finder_provider)
-    required_args = inspect.getargs(preparer_fn.__init__.__code__).args  # type: ignore
+    required_args, required_kwargs = get_allowed_args(preparer_fn)  # type: ignore
     if not req_tracker and not req_tracker_fn and "req_tracker" in required_args:
         raise TypeError("No requirement tracker and no req tracker generator found!")
     if "downloader" in required_args and not downloader_provider:
@@ -897,6 +933,38 @@ def make_preparer(
         yield result
 
 
+@contextlib.contextmanager
+def _ensure_wheel_cache(
+    wheel_cache=None,  # type: Optional[Type[TWheelCache]]
+    wheel_cache_provider=None,  # type: Optional[Callable]
+    format_control=None,  # type: Optional[TFormatControl]
+    format_control_provider=None,  # type: Optional[Type[TShimmedFunc]]
+    options=None,  # type: Optional[Values]
+    cache_dir=None,  # type: Optional[str]
+):
+    if wheel_cache is not None:
+        yield wheel_cache
+    elif wheel_cache_provider is not None:
+        with ExitStack() as stack:
+            cache_dir = getattr(options, "cache_dir", cache_dir)
+            format_control = getattr(
+                options,
+                "format_control",
+                format_control_provider(None, None),  # TFormatControl
+            )
+            wheel_cache = stack.enter_context(
+                wheel_cache_provider(cache_dir, format_control)
+            )
+            yield wheel_cache
+
+
+def get_ireq_output_path(wheel_cache, ireq):
+    if getattr(wheel_cache, "get_path_for_link", None):
+        return wheel_cache.get_path_for_link(ireq.link)
+    elif getattr(wheel_cache, "cached_wheel", None):
+        return wheel_cache.cached_wheel(ireq.link, ireq.name).url_without_fragment
+
+
 def get_resolver(
     resolver_fn,  # type: TShimmedFunc
     install_req_provider=None,  # type: Optional[TShimmedFunc]
@@ -917,6 +985,7 @@ def get_resolver(
     make_install_req=None,  # type: Optional[Callable]
     install_cmd_provider=None,  # type: Optional[TShimmedFunc]
     install_cmd=None,  # type: Optional[TCommandInstance]
+    use_pep517=True,  # type: bool
 ):
     # (...) -> TResolver
     """
@@ -964,6 +1033,7 @@ def get_resolver(
         to the resolver for actually generating install requirements, if necessary
     :param Optional[TCommandInstance] install_cmd: The install command used to create
         the finder, session, and options if needed, defaults to None.
+    :param bool use_pep517: Whether to use the pep517 build process.
     :return: A new resolver instance.
     :rtype: :class:`~pip._internal.legacy_resolve.Resolver`
 
@@ -1022,13 +1092,13 @@ def get_resolver(
         assert isinstance(install_cmd_provider, (type, functools.partial))
         install_cmd = install_cmd_provider()
     if options is None and install_cmd is not None:
-        options = install_cmd.parser.parse_args([])  # type: ignore
+        options, _ = install_cmd.parser.parse_args([])  # type: ignore
     for arg, val in install_cmd_dependency_map.items():
         if arg not in required_args:
             continue
         elif val is None and install_cmd is None:
             raise TypeError(
-                "Preparer requires a {0} but did not receive one "
+                "Preparer requires a {} but did not receive one "
                 "and cannot generate one".format(arg)
             )
         elif arg == "session" and val is None:
@@ -1038,26 +1108,21 @@ def get_resolver(
         resolver_kwargs[arg] = val
     if "make_install_req" in required_args:
         if make_install_req is None and install_req_provider is not None:
+            make_install_req_kwargs = {
+                "isolated": isolated,
+                "wheel_cache": wheel_cache,
+                "use_pep517": use_pep517,
+            }
+            factory_args, factory_kwargs = filter_allowed_args(
+                install_req_provider, **make_install_req_kwargs
+            )
             make_install_req = functools.partial(
-                install_req_provider,
-                isolated=isolated,
-                wheel_cache=wheel_cache,
-                # use_pep517=use_pep517,
+                install_req_provider, *factory_args, **factory_kwargs
             )
         assert make_install_req is not None
         resolver_kwargs["make_install_req"] = make_install_req
     if "isolated" in required_args:
         resolver_kwargs["isolated"] = isolated
-    if "wheel_cache" in required_args:
-        if wheel_cache is None and wheel_cache_provider is not None:
-            cache_dir = getattr(options, "cache_dir", None)
-            format_control = getattr(
-                options,
-                "format_control",
-                format_control_provider(None, None),  # type: ignore
-            )
-            wheel_cache = wheel_cache_provider(cache_dir, format_control)
-        resolver_kwargs["wheel_cache"] = wheel_cache
     resolver_kwargs.update(
         {
             "upgrade_strategy": upgrade_strategy,
@@ -1069,10 +1134,19 @@ def get_resolver(
             "preparer": preparer,
         }
     )
+    if "wheel_cache" in required_args:
+        with _ensure_wheel_cache(
+            wheel_cache=wheel_cache,
+            wheel_cache_provider=wheel_cache_provider,
+            format_control_provider=format_control_provider,
+            options=options,
+        ) as wheel_cache:
+            resolver_kwargs["wheel_cache"] = wheel_cache
+            return resolver_fn(**resolver_kwargs)  # type: ignore
     return resolver_fn(**resolver_kwargs)  # type: ignore
 
 
-def resolve(
+def resolve(  # noqa:C901
     ireq,  # type: TInstallRequirement
     reqset_provider=None,  # type: Optional[TShimmedFunc]
     req_tracker_provider=None,  # type: Optional[TShimmedFunc]
@@ -1102,6 +1176,7 @@ def resolve(
     wheel_download_dir=None,  # type: Optional[str]
     wheel_cache=None,  # type: Optional[TWheelCache]
     require_hashes=None,  # type: bool
+    check_supported_wheels=True,  # type: bool
 ):
     # (...) -> Set[TInstallRequirement]
     """
@@ -1163,6 +1238,8 @@ def resolve(
     :param Optional[TWheelCache] wheel_cache: The wheel cache to use, defaults to None
     :param bool require_hashes: Whether to require hashes when resolving. Defaults to
         False.
+    :param bool check_supported_wheels: Whether to check support of wheels before including
+        them in resolution.
     :return: A dictionary mapping requirements to corresponding
         :class:`~pip._internal.req.req_install.InstallRequirement`s
     :rtype: :class:`~pip._internal.req.req_install.InstallRequirement`
@@ -1231,20 +1308,29 @@ def resolve(
         if session is None:
             session = get_session(install_cmd=install_command, options=options)
         if finder is None:
-            finder = finder_provider(install_command, options=options, session=session)  # type: ignore
+            finder = finder_provider(
+                install_command, options=options, session=session
+            )  # type: ignore
         format_control = getattr(options, "format_control", None)
         if not format_control:
             format_control = format_control_provider(None, None)  # type: ignore
-        wheel_cache = wheel_cache_provider(
-            kwargs["cache_dir"], format_control
+        wheel_cache = ctx.enter_context(
+            wheel_cache_provider(kwargs["cache_dir"], format_control)
         )  # type: ignore
         ireq.is_direct = True  # type: ignore
-        ireq.build_location(kwargs["build_dir"])  # type: ignore
+        build_location_kwargs = {"build_dir": kwargs["build_dir"], "autodelete": True}
+        call_function_with_correct_args(ireq.build_location, **build_location_kwargs)
         if reqset_provider is None:
             raise TypeError(
                 "cannot resolve without a requirement set provider... failed!"
             )
-        reqset = reqset_provider(install_command, options=options, session=session, wheel_download_dir=wheel_download_dir, **kwargs)  # type: ignore
+        reqset = reqset_provider(
+            install_command,
+            options=options,
+            session=session,
+            wheel_download_dir=wheel_download_dir,
+            **kwargs
+        )  # type: ignore
         if getattr(reqset, "prepare_files", None):
             reqset.add_requirement(ireq)
             results = reqset.prepare_files(finder)
@@ -1267,7 +1353,6 @@ def resolve(
             "use_user_site": use_user_site,
             "require_hashes": require_hashes,
         }
-        # with req_tracker_provider() as req_tracker:
         if isinstance(req_tracker_provider, (types.FunctionType, functools.partial)):
             preparer_args["req_tracker"] = ctx.enter_context(req_tracker_provider())
         resolver_keys = [
@@ -1292,15 +1377,27 @@ def resolve(
             wheel_cache=wheel_cache,
             **resolver_args
         )  # type: ignore
-        reqset.add_requirement(ireq)
         resolver.require_hashes = kwargs.get("require_hashes", False)  # type: ignore
-        resolver.resolve(reqset)  # type: ignore
-        results = reqset.requirements
-        reqset.cleanup_files()
+        _, required_resolver_args = get_method_args(resolver.resolve)
+        resolver_args = []
+        if "requirement_set" in required_resolver_args.args:
+            reqset.add_requirement(ireq)
+            resolver_args.append(reqset)
+        elif "root_reqs" in required_resolver_args.args:
+            resolver_args.append([ireq])
+        if "check_supported_wheels" in required_resolver_args.args:
+            resolver_args.append(check_supported_wheels)
+        result_reqset = resolver.resolve(*resolver_args)  # type: ignore
+        if result_reqset is None:
+            result_reqset = reqset
+        results = result_reqset.requirements
+        cleanup_fn = getattr(reqset, "cleanup_files", None)
+        if cleanup_fn is not None:
+            cleanup_fn()
         return results
 
 
-def build_wheel(
+def build_wheel(  # noqa:C901
     req=None,  # type: Optional[TInstallRequirement]
     reqset=None,  # type: Optional[Union[TReqSet, Iterable[TInstallRequirement]]]
     output_dir=None,  # type: Optional[str]
@@ -1330,8 +1427,9 @@ def build_wheel(
     build_many_provider=None,  # type: Optional[TShimmedFunc]
     install_command_provider=None,  # type: Optional[TShimmedFunc]
     finder_provider=None,  # type: Optional[TShimmedFunc]
+    reqset_provider=None,  # type: Optional[TShimmedFunc]
 ):
-    # type: (...) -> Optional[Union[str, Tuple[List[TInstallRequirement], List[TInstallRequirement]]]]
+    # type: (...) -> Generator[Union[str, Tuple[List[TInstallRequirement], ...]], None, None]
     """
     Build a wheel or a set of wheels
 
@@ -1383,19 +1481,21 @@ def build_wheel(
     :param Optional[TShimmedFunc] install_command_provider: A shim for providing new
         install command instances
     :param TShimmedFunc finder_provider: A provider to package finder instances
+    :param TShimmedFunc reqset_provider: A provider for requirement set generation
     :return: A tuple of successful and failed install requirements or else a path to
         a wheel
     :rtype: Optional[Union[str, Tuple[List[TInstallRequirement], List[TInstallRequirement]]]]
     """
     wheel_cache_provider = resolve_possible_shim(wheel_cache_provider)
-    preparer = resolve_possible_shim(preparer)
+    preparer_provider = resolve_possible_shim(preparer_provider)
     wheel_builder_provider = resolve_possible_shim(wheel_builder_provider)
     build_one_provider = resolve_possible_shim(build_one_provider)
     build_one_inside_env_provider = resolve_possible_shim(build_one_inside_env_provider)
     build_many_provider = resolve_possible_shim(build_many_provider)
     install_cmd_provider = resolve_possible_shim(install_command_provider)
     format_control_provider = resolve_possible_shim(format_control_provider)
-    finder_provider = resolve_possible_shim(finder_provider)
+    finder_provider = resolve_possible_shim(finder_provider) or get_package_finder
+    reqset_provider = resolve_possible_shim(reqset_provider)
     global_options = [] if global_options is None else global_options
     build_options = [] if build_options is None else build_options
     options = None
@@ -1409,25 +1509,31 @@ def build_wheel(
     }
     if not req and not reqset:
         raise TypeError("Must provide either a requirement or requirement set to build")
-    if wheel_cache is None and (reqset is not None or output_dir is None):
-        if install_command is None:
-            assert isinstance(install_cmd_provider, (type, functools.partial))
-            install_command = install_cmd_provider()
-        kwargs, options = populate_options(install_command, options, **kwarg_map)
-        format_control = getattr(options, "format_control", None)
-        if not format_control:
-            format_control = format_control_provider(None, None)  # type: ignore
-        wheel_cache = wheel_cache_provider(options.cache_dir, format_control)
-    if req and not reqset and not output_dir:
-        output_dir = wheel_cache.get_path_for_link(req.link)
-    if not reqset and build_one_provider:
-        yield build_one_provider(req, output_dir, build_options, global_options)
-    elif build_many_provider:
-        yield build_many_provider(
-            reqset, wheel_cache, build_options, global_options, check_binary_allowed
-        )
-    else:
-        with ExitStack() as ctx:
+    with ExitStack() as ctx:
+        kwargs = kwarg_map.copy()
+        if wheel_cache is None and (reqset is not None or output_dir is None):
+            if install_command is None:
+                assert isinstance(install_cmd_provider, (type, functools.partial))
+                install_command = install_cmd_provider()
+            kwargs, options = populate_options(install_command, options, **kwarg_map)
+            format_control = getattr(options, "format_control", None)
+            if not format_control:
+                format_control = format_control_provider(None, None)  # type: ignore
+            wheel_cache = ctx.enter_context(
+                wheel_cache_provider(options.cache_dir, format_control)
+            )
+        if req and not reqset and not output_dir:
+            output_dir = get_ireq_output_path(wheel_cache, req)
+        if not reqset and build_one_provider:
+            yield build_one_provider(req, output_dir, build_options, global_options)
+        elif build_many_provider:
+            yield build_many_provider(
+                reqset, wheel_cache, build_options, global_options, check_binary_allowed
+            )
+        else:
+            builder_args, builder_kwargs = get_allowed_args(wheel_builder_provider)
+            if "requirement_set" in builder_args and not reqset:
+                reqset = reqset_provider()
             if session is None and finder is None:
                 session = get_session(install_cmd=install_command, options=options)
                 finder = finder_provider(
@@ -1465,7 +1571,7 @@ def build_wheel(
             )
             if req and not reqset:
                 if not output_dir:
-                    output_dir = wheel_cache.get_path_for_link(req.link)
+                    output_dir = get_ireq_output_path(wheel_cache, req)
                 if use_pep517 is not None:
                     req.use_pep517 = use_pep517
                 yield builder._build_one(req, output_dir)
